#include "ShaderConstants.fxh"

struct VS_Input {
	float3 position : POSITION;
	float4 color : COLOR;
	float2 uv0 : TEXCOORD_0;
	float2 uv1 : TEXCOORD_1;
	float3 IVGWPos : POSITION;
	float3 wpos : POSITION;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
};


struct PS_Input {
	float4 position : SV_Position;
#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
	float3 IVGWPos : WIVG;
	float3 wpos : WPOS;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};


static const float rA = 1.0;
static const float rB = 1.0;
static const float3 UNIT_Y = float3(0, 1, 0);
static const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange


ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput)
{
#ifndef BYPASS_PIXEL_SHADER
	PSInput.uv0 = VSInput.uv0;
	PSInput.uv1 = VSInput.uv1;
	PSInput.color = VSInput.color;
#endif

bool night = (FOG_COLOR.r<0.15 && FOG_COLOR.g<0.15);
bool inRain = (FOG_CONTROL.x < 0.55 && FOG_CONTROL.x > 0.1);
bool greenPlant = (PSInput.color.g + PSInput.color.g > PSInput.color.r + PSInput.color.b);
bool water = (PSInput.color.b > PSInput.color.r);
bool nether = (FOG_COLOR.r>FOG_COLOR.b && FOG_COLOR.r<0.5 && FOG_COLOR.b<0.2);
bool TheEnd = (FOG_COLOR.r > FOG_COLOR.g && FOG_COLOR.b > FOG_COLOR.g && FOG_COLOR.r < 0.05 && FOG_COLOR.b < 0.05 && FOG_COLOR.g < 0.05);

#ifdef AS_ENTITY_RENDERER
	#ifdef INSTANCEDSTEREO
		int i = VSInput.instanceID;
		PSInput.position = mul(WORLDVIEWPROJ_STEREO[i], float4(VSInput.position, 1));
	#else
		PSInput.position = mul(WORLDVIEWPROJ, float4(VSInput.position, 1));
	#endif
		float3 worldPos = PSInput.position;
#else
		float3 worldPos = (VSInput.position.xyz * CHUNK_ORIGIN_AND_SCALE.w) + CHUNK_ORIGIN_AND_SCALE.xyz;
	
		// Transform to view space before projection instead of all at once to avoid floating point errors
		// Not required for entities because they are already offset by camera translation before rendering
		// World position here is calculated above and can get huge
	#ifdef INSTANCEDSTEREO
		int i = VSInput.instanceID;
	
		PSInput.position = mul(WORLDVIEW_STEREO[i], float4(worldPos, 1 ));
		PSInput.position = mul(PROJ_STEREO[i], PSInput.position);
	
	#else
		PSInput.position = mul(WORLDVIEW, float4( worldPos, 1 ));
		PSInput.position = mul(PROJ, PSInput.position);
	#endif

#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
		PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
		PSInput.renTarget_id = VSInput.instanceID;
#endif
///// find distance from the camera

#if defined(FOG) || defined(BLEND)
	#ifdef FANCY
		float3 relPos = -worldPos;
		float cameraDepth = length(relPos);
	#else
		float cameraDepth = PSInput.position.z;
	#endif
#endif
if(PSInput.color.g < 0.61 && PSInput.color.a!=0.0){if(PSInput.uv1.y > 0.875){float3 shdd = float3(0.0,0.0,0.1)+PSInput.uv1.x*(0.0,0.0,-0.1);
PSInput.color.rgb += shdd;
if(night==true){if(nether==false){
float3 shdn = float3(-0.1,-0.1,0.2)+PSInput.uv1.x*float3(0.1,0.1,-0.2);
PSInput.color.rgb += shdn;}}}}
if(nether==false){if(FOG_CONTROL.x==0.0){PSInput.position.x += sin(TIME*3.6+worldPos.x+worldPos.z+worldPos.x+worldPos.z+worldPos.y)*sin(worldPos.z)*0.066;}}
#ifdef ALPHA_TEST
   #ifdef FANCY
if(inRain==false){
if(greenPlant==true){float wavespeed1 = 3.6; PSInput.position.xy += sin(TOTAL_REAL_WORLD_TIME*wavespeed1+VSInput.position.x+VSInput.position.z+VSInput.position.x+VSInput.position.z+VSInput.position.y)*sin(VSInput.position.z)*0.066;}}
if(inRain==true){
if(greenPlant==true){float wavespeed2 = 4.3; PSInput.position.xy += sin(TOTAL_REAL_WORLD_TIME*wavespeed2+VSInput.position.x+VSInput.position.z+VSInput.position.x+VSInput.position.z+VSInput.position.y)*sin(VSInput.position.z)*0.076;}}
   #endif
#endif
#ifdef BLEND
if(water==true){if(PSInput.color.a<0.95){PSInput.position.y += sin(TIME*3.6+worldPos.x+worldPos.z+worldPos.x+worldPos.z+worldPos.y)*sin(worldPos.z)*0.078;}}
#endif
#ifdef BLEND
if(water==true){if(PSInput.color.a<0.95){float wavespeed1 = 3.6; PSInput.position.x += sin(TOTAL_REAL_WORLD_TIME*wavespeed1+worldPos.x+worldPos.z+worldPos.x+worldPos.z+worldPos.y)*sin(worldPos.z)*0.05;}}
#endif

#ifndef BYPASS_PIXEL_SHADER
	PSInput.IVGWPos = VSInput.position.xyz;
	PSInput.wpos = worldPos.xyz;
#endif

	///// apply fog

#ifdef FOG
	float len = cameraDepth / RENDER_DISTANCE;
#ifdef ALLOW_FADE
	len += RENDER_CHUNK_FOG_ALPHA.r;
#endif

    PSInput.fogColor.rgb = FOG_COLOR.rgb;
	PSInput.fogColor.a = clamp((len - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 0.0, 1.0);

#endif

///// blended layer (mostly water) magic
#ifdef BLEND
	//Mega hack: only things that become opaque are allowed to have vertex-driven transparency in the Blended layer...
	//to fix this we'd need to find more space for a flag in the vertex format. color.a is the only unused part
	bool OpaqueIfFar = VSInput.color.a < 0.95;
	if(OpaqueIfFar) {
		#ifdef FANCY  /////enhance water
			float cameraDist = cameraDepth / FAR_CHUNKS_DISTANCE;
		#else
			float3 relPos = -worldPos.xyz;
			float camDist = length(relPos);
			float cameraDist = camDist / FAR_CHUNKS_DISTANCE;
		#endif //FANCY
		
		float alphaFadeOut = clamp(cameraDist, 0.0, 1.0);
		PSInput.color.a = lerp(VSInput.color.a, 1.0, alphaFadeOut);
	}
#endif
}
///PXD