#include "ShaderConstants.fxh"
#include "util.fxh"
#define N pow(max(min(1.0-FOG_COLOR.r*1.5,1.0),0.0),1.2)
#define D pow(max(min(1.0-FOG_COLOR.b*1.2,1.0),0.0),0.5)
#define R (1.0-pow(FOG_CONTROL.y,11.0))

struct PS_Input
{
    float4 position : SV_Position;
    float2 uv : TEXCOORD_0_FB_MSAA;
	float3 xyz : XYZ;
};


struct PS_Output
{
    float4 color : SV_Target;
};

float2 hash( float2 p )
{
	p = float2( dot(p,float2(127.1,311.7)),
			  dot(p,float2(269.5,183.3)) );

	return -1.0 + 2.0*frac(sin(p)*43758.5453123);
}

float noise( in float2 p )
{
     float K1 = 0.366025404;
     float K2 = 0.211324865;

	float2 i = floor( p + (p.x+p.y)*K1 );
	
    float2 a = p - i + (i.x+i.y)*K2;
    float2 o = step(a.yx,a.xy);    
    float2 b = a - o + K2;
	float2 c = a - 1.0 + 2.0*K2;

    float3 h = max( 0.5-float3(dot(a,a), dot(b,b), dot(c,c) ), 0.0 );

	float3 n = h*h*h*h*float3( dot(a,hash(i+0.0)), dot(b,hash(i+o)), dot(c,hash(i+1.0)));

    return dot( n, 70.0 );
	
}

float4 cirrus( float2 p) {

float2 uv = p*0.8;
	uv.x += TIME * 0.05;

	uv.y *= 0.75;
	float n  = noise(uv * float2(0.25, 1.0)) * 0.5;
		uv += float2(n * 0.5, 0.3) * 1.2; uv *= 2.50;
		  n += noise(uv) * 0.25;
		uv += float2(n * 1.2, 0.2) * -1.2+ float2(TIME * 0.01, 0.2); uv *= 3.01;
		  n += noise(uv) * 0.105;
		uv += float2(n * 0.4, 0.1) *1.0 + float2(TIME * 0.03, 0.1); uv *= 3.02;
		  n += noise(uv) * 0.06;
	n = smoothstep(0.0, 0.8, n);

	return n;
}


float4 Sky( in float3 rd )
{

    rd.y = max(rd.y, 0.);
    
    float3 col = 0.0;
    
	
	float3 sunglow = float3(.5, .5 - exp( -rd.y*20. )*.3, .0) * exp(-rd.y*9.);
	sunglow = lerp(sunglow,FOG_COLOR.rgb,R);
	
	float3 dayglow = FOG_COLOR.rgb * 1.2 * exp(-rd.y*9.);
	dayglow = lerp(dayglow,FOG_COLOR.rgb,R);
	
	float3 nightglow = float3(0.0,0.12,0.3)*0.65;
	nightglow = lerp(nightglow,FOG_COLOR.rgb,R);
	
	float3 glow = lerp(lerp(dayglow,sunglow,D),nightglow,N);
	
	float3 day = float3(.3, .5, .7)*1.5;

	float3 night = 0.001;
	
    col +=glow; 
	
    col += lerp(day,night,N) * (1. - exp(-rd.y*8.) ) * exp(-rd.y*.9) ; 
	
    
    return float4(pow(col,1.4),1.0);
}

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
 float l = FOG_COLOR.r + FOG_COLOR.g + FOG_COLOR.b;
 float3 BASE1_DAY =1.0, BASE2_DAY = float3(1.0,0.95,0.55)*0.5;
 float3 BASE1_SUNSET = float3(1.2,0.3,0.05), BASE2_SUNSET = FOG_COLOR.rgb;
 float3 BASE1_NIGHT = float3(0.115,0.12,0.125)*0.5, BASE2_NIGHT = float3(0.115,0.12,0.125)*0.3;
 float3 BASE1_RAIN = lerp(0.4,0.075,N), BASE2_RAIN = lerp(0.4,0.075,N);

 float3 BASE1 = lerp(lerp(BASE1_DAY,BASE1_SUNSET,D),BASE1_NIGHT,N);
 BASE1 = lerp(BASE1,BASE1_RAIN,R);


	float3 dir = (PSInput.xyz.xyz/(PSInput.xyz.y-0.15));

    float3 rd = -PSInput.xyz;
	rd.x *= length(rd);
	rd.y += 0.15;


	
	float4 sky = Sky(rd);
	
	float4 clouds2 = (float4(BASE1,1.0)*cirrus(dir.xz)*(1.0-sky))*0.5;

    clouds2 = lerp(clouds2,0.0,1.-exp(min(rd.y, 0.)*100.));
    
    PSOutput.color = lerp(sky+clouds2,0.0,1.-exp(min(rd.y, 0.)*100.));

}
