#include "ShaderConstants.fxh"
#define N pow(max(min(1.0-FOG_COLOR.r*1.5,1.0),0.0),1.2)
#define D pow(max(min(1.0-FOG_COLOR.b*1.2,1.0),0.0),0.5)
#define R (1.0-pow(FOG_CONTROL.y,11.0))

#define DAY_R 0.85	
#define DAY_G 0.85
#define DAY_B 0.85
#define DAY float3(DAY_R,DAY_G,DAY_B)


#define DAYDAYSHADOW_R 0.3675
#define DAYDAYSHADOW_G 0.4095
#define DAYDAYSHADOW_B 0.525
#define DAYSHADOW float3(DAYDAYSHADOW_R,DAYDAYSHADOW_G,DAYDAYSHADOW_B)


#define NIGHT_R 0.05
#define NIGHT_G 0.1
#define NIGHT_B 0.45
#define NIGHT float3(NIGHT_R,NIGHT_G,NIGHT_B)*0.2


#define TORCH_R 5.0
#define TORCH_G 2.0
#define TORCH_B -1.0
#define TORCH_DAY float3(TORCH_R,TORCH_G,TORCH_B)
#define TORCH_NIGHT TORCH_DAY*40.0

#define TINTFOG_R 0.55
#define TINTFOG_G 0.6
#define TINTFOG_B 0.8
#define TINTFOG float3(TINTFOG_R,TINTFOG_G,TINTFOG_B)*1.8

#include "util.fxh"

struct PS_Input
{
	float4 position : SV_Position;
	float3 V_xyz : V_XYZ;
	float3 W_xyz : W_XYZ;
	float  fp : FP;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;

};

ROOT_SIGNATURE


float3 toneMap(float3 color, float brig)
{
	 float A = 2.3f;
	 float B = 0.24f;
	 float C = 1.83f;

	color *= brig;
	
	return (color * (A * color)) / (color * (C * color) + B);
}

void main(in PS_Input PSInput, out PS_Output PSOutput)
{

float2 uv = PSInput.V_xyz.xz/5.0;

float3 ori = float3(0.0,5.0,0.0);
float3 eye = normalize(float3(uv.x,-1.0,uv.y)); 
float3 rd =  normalize(float3(uv.x/4.0, 1.0,uv.y/4.0));

float3 WORLDNORMAL = normalize(cross(ddx(PSInput.W_xyz),ddy(PSInput.W_xyz)));

#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif


#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

	
float3 SourceColor = sqrt(PSInput.color.rgb);

#ifdef ALPHA_TEST
if(PSInput.color.g + PSInput.color.g > PSInput.color.r + PSInput.color.b){
SourceColor = PSInput.color*1.1;
if(PSInput.color.a < 0.95) {
SourceColor = 0.0;
}
}
#endif
if(PSInput.color.g + PSInput.color.g > PSInput.color.r + PSInput.color.b){
SourceColor = PSInput.color*1.1;
}

	diffuse.rgb *= SourceColor;

#ifdef BLEND
if(PSInput.color.a < 0.95) {
#ifdef FANCY  

diffuse = PSInput.color.a;

#endif
}
#endif

	
#else
	float2 u = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, u).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif



float UV_X = 1.0;


float UV_Y =(PSInput.uv1.y*1.03-0.874244)*40.0;
float SHADOW_MASK = max(min(UV_Y,UV_X),0.0);

diffuse.rgb = pow(diffuse.rgb,1.2);

float3 LIGHT =lerp(DAY,NIGHT,N);
float3 SHADOW =lerp(DAYSHADOW,NIGHT,N);
LIGHT = lerp(LIGHT,SHADOW,R);

diffuse.rgb = toneMap(diffuse.rgb,1.2);
diffuse.rgb *= lerp(SHADOW,LIGHT,SHADOW_MASK);

float3 TORCH = lerp(TORCH_DAY,TORCH_NIGHT,N);



diffuse.rgb = lerp(diffuse.rgb,diffuse.rgb*TORCH,pow(PSInput.uv1.x*0.825,5.5));


float FOG_MASK = lerp(min(max(PSInput.fp*0.55 ,0.0),0.4),min(max(PSInput.fp*0.25 ,0.0),0.75),SHADOW_MASK);

diffuse.rgb =lerp(diffuse.rgb,FOG_COLOR.rgb*pow(TINTFOG,1.1),FOG_MASK);




	PSOutput.color = diffuse;



#endif 
}